from playwright.async_api import Page as PageAsync
from playwright.sync_api import Page as PageSync
from selenium.webdriver.remote.webdriver import WebDriver

from ._base import BrowserDriver
from .playwright import PlaywrightAsyncDriver, PlaywrightSyncDriver
from .selenium import SeleniumDriver
from .types import AnyDriver

# TODO: make selenium and playwright drivers optional


def driver_factory(driver: AnyDriver) -> BrowserDriver:
    if isinstance(driver, WebDriver):
        return SeleniumDriver(driver)
    elif isinstance(driver, PageSync):
        return PlaywrightSyncDriver(driver)
    elif isinstance(driver, PageAsync):
        return PlaywrightAsyncDriver(driver)
    # TODO: add support for Puppeteer

    else:
        raise ValueError(
            "Invalid driver type: please provide a Selenium WebDriver or a Playwright Page"
        )


__all__ = [
    "BrowserDriver",
    "PlaywrightAsyncDriver",
    "PlaywrightSyncDriver",
    "SeleniumDriver",
    "driver_factory",
    "AnyDriver",
]



import os
from typing import Dict, Tuple

from tarsier._base import ITarsier
from tarsier.driver import BrowserDriver, SeleniumDriver
from tarsier.ocr import OCRService

TagToXPath = Dict[int, str]


class Tarsier(ITarsier):
    _JS_TAG_UTILS = os.path.dirname(os.path.realpath(__file__)) + "/tag_utils.js"

    def __init__(self, ocr_service: OCRService):
        self._ocr_service = ocr_service
        with open(self._JS_TAG_UTILS, "r") as f:
            self._js_utils = f.read()

    async def page_to_image(
        self, driver: BrowserDriver
    ) -> Tuple[bytes, Dict[int, str]]:
        tag_to_xpath = await self._tag_page(driver)

        screenshot = await self._take_screenshot(driver)
        return screenshot, tag_to_xpath

    async def page_to_text(self, driver: BrowserDriver) -> Tuple[str, TagToXPath]:
        image, tag_to_xpath = await self.page_to_image(driver)
        page_text = self._run_ocr(image)
        return page_text, tag_to_xpath

    @staticmethod
    async def _take_screenshot(driver: BrowserDriver) -> bytes:
        # TODO: scroll & stitch here, don't do viewport resizing
        script = "() => [window.innerWidth, window.innerHeight, document.documentElement.scrollHeight];"
        if isinstance(driver, SeleniumDriver):
            script = f"return [window.innerWidth, window.innerHeight, document.documentElement.scrollHeight];"

        default_width, default_height, content_height = await driver.run_js(script)

        await driver.set_viewport_size(default_width, content_height)
        screenshot = await driver.take_screenshot()
        await driver.set_viewport_size(default_width, default_height)

        return screenshot

    def _run_ocr(self, image: bytes) -> str:
        ocr_text = self._ocr_service.annotate(image)
        page_text = self._ocr_service.format_text(ocr_text)
        return page_text

    async def _tag_page(self, driver: BrowserDriver) -> Dict[int, str]:
        await driver.run_js(self._js_utils)

        script = "tagifyWebpage(null, null);"
        if isinstance(driver, SeleniumDriver):
            script = f"return window.{script}"

        _, tag_to_xpath = await driver.run_js(script)
        return {int(key): value for key, value in tag_to_xpath.items()}



import asyncio
from typing import Any

from selenium.webdriver.remote.webdriver import WebDriver

from tarsier.driver._base import BrowserDriver


class SeleniumDriver(BrowserDriver):
    def __init__(self, driver: WebDriver):
        self.driver = driver

    async def run_js(self, js: str) -> Any:
        return self.driver.execute_script(js)

    async def take_screenshot(self) -> bytes:
        return self.driver.get_screenshot_as_png()

    async def set_viewport_size(self, width: int, height: int) -> None:
        self.driver.set_window_size(width, height)



from asyncio import Protocol
from typing import Dict, Tuple

from tarsier.driver import AnyDriver, BrowserDriver, driver_factory


class ITarsier(Protocol):
    async def page_to_image(
        self, driver: BrowserDriver
    ) -> Tuple[bytes, Dict[int, str]]:
        raise NotImplementedError()

    async def page_to_text(self, driver: BrowserDriver) -> Tuple[str, Dict[int, str]]:
        raise NotImplementedError()

    @staticmethod
    def create_driver(driver: AnyDriver) -> BrowserDriver:
        return driver_factory(driver)
